pro part_b, scipix_n, refpix_r, search_tag, out_file

; Part_B computes the alpha and beta gain coefficients from sums of 
; n*n n*p n*r p*p p*r r*r that were generated by repeated runs of part_A from 
; many ramps. These coefficients can be applied using part_C.

; INPUT:
; search_tag = string with wildcard to identify the frames (FITS files) to be processed.
; OPTIONAL INPUT:
; out_file   = output filename [default = 'part_B.xdr'] 

; OUTPUT is saved in "part_B.xdr" in the current working directory.
;-------------------------------------------------------------------------------
;files = file_search('*N16R4*_1_[1234]_2011*/E00*/*sums.xdr',count=nf)
;files = file_search('*N16R4*_2011*/E*/*sums.xdr',count=nf)

; Changes:
; v2: 2012/2/13: added _0alpha results for the avaluation of reference output only. 


if (n_elements(out_file) ne 1) then out_file= 'part_B.xdr'

files = file_search(search_tag,count=nf)

restore,files[0]

for i=1,nf-1 do begin
  SUM_NN_0 += restorex(files[i],'SUM_NN_0')
  SUM_NN_1 += restorex(files[i],'SUM_NN_1')
  SUM_NN_2 += restorex(files[i],'SUM_NN_2')
  SUM_NN_3 += restorex(files[i],'SUM_NN_3')
  SUM_NO_0 += restorex(files[i],'SUM_NO_0')
  SUM_NO_1 += restorex(files[i],'SUM_NO_1')
  SUM_NO_2 += restorex(files[i],'SUM_NO_2')
  SUM_NO_3 += restorex(files[i],'SUM_NO_3')
  SUM_NR_0 += restorex(files[i],'SUM_NR_0')
  SUM_NR_1 += restorex(files[i],'SUM_NR_1')
  SUM_NR_2 += restorex(files[i],'SUM_NR_2')
  SUM_NR_3 += restorex(files[i],'SUM_NR_3')
  SUM_OO   += restorex(files[i],'SUM_OO')
  SUM_RO_0 += restorex(files[i],'SUM_RO_0')
  SUM_RO_1 += restorex(files[i],'SUM_RO_1')
  SUM_RO_2 += restorex(files[i],'SUM_RO_2')
  SUM_RO_3 += restorex(files[i],'SUM_RO_3')
  SUM_RR_0 += restorex(files[i],'SUM_RR_0')
  SUM_RR_1 += restorex(files[i],'SUM_RR_1')
  SUM_RR_2 += restorex(files[i],'SUM_RR_2')
  SUM_RR_3 += restorex(files[i],'SUM_RR_3')
  print,i,nf,string(13b),form='(i," of ",i,a,$)'
endfor
print

nsum = n_elements(SUM_OO)
f5 = dist(nsum,1)/nsum*1e5

; construct the refereence pixel power filter [1,cos,0,cos,1]
;filter_r = replicate(1.,nsum) 
;scipix_n = 32
;refpix_r = 8
pad = 8 
row = 512L+512L/scipix_n*(refpix_r+2)+pad
s = [0,0,2048]
elen=110000L/(scipix_n+refpix_r+2)    ; length of apodization cosine filter
blen=(512L+512L/scipix_n*(refpix_r+2)+pad)/(scipix_n+refpix_r+2)*s[2]/2-elen/2 ; max unfiltered frequency 
a=(cos(findgen(elen)*!pi/elen)+1)/2 
a=[replicate(1.,blen),a,fltarr(row*s[2]/2-2*blen-2*elen),reverse(a),replicate(1.,blen)]
a=[a,reverse(shift(a,-1))]
filter_r = a
 
alpha_0 = filter_r*(sum_oo*sum_nr_0/sum_rr_0-sum_no_0*sum_ro_0/sum_rr_0)/(sum_oo-filter_r^2*sum_ro_0^2/sum_rr_0)
beta_0 = (sum_no_0-filter_r^2*sum_nr_0*sum_ro_0/sum_rr_0)/(sum_oo-filter_r^2*sum_ro_0^2/sum_rr_0)
alpha_1 = filter_r*(sum_oo*sum_nr_1/sum_rr_1-sum_no_1*sum_ro_1/sum_rr_1)/(sum_oo-filter_r^2*sum_ro_1^2/sum_rr_1)
beta_1 = (sum_no_1-filter_r^2*sum_nr_1*sum_ro_1/sum_rr_1)/(sum_oo-filter_r^2*sum_ro_1^2/sum_rr_1)
alpha_2 = filter_r*(sum_oo*sum_nr_2/sum_rr_2-sum_no_2*sum_ro_2/sum_rr_2)/(sum_oo-filter_r^2*sum_ro_2^2/sum_rr_2)
beta_2 = (sum_no_2-filter_r^2*sum_nr_2*sum_ro_2/sum_rr_2)/(sum_oo-filter_r^2*sum_ro_2^2/sum_rr_2)
alpha_3 = filter_r*(sum_oo*sum_nr_3/sum_rr_3-sum_no_3*sum_ro_3/sum_rr_3)/(sum_oo-filter_r^2*sum_ro_3^2/sum_rr_3)
beta_3 = (sum_no_3-filter_r^2*sum_nr_3*sum_ro_3/sum_rr_3)/(sum_oo-filter_r^2*sum_ro_3^2/sum_rr_3)

save,file=out_file,files,f5,filter_r,alpha_0,beta_0,alpha_1,beta_1,alpha_2,beta_2,alpha_3,beta_3

; Now calculate coefficients for reference pixels only
alpha_0 = filter_r*(sum_nr_0/sum_rr_0)
beta_0 = 0
alpha_1 = filter_r*(sum_nr_1/sum_rr_1)
beta_1 = 0
alpha_2 = filter_r*(sum_nr_2/sum_rr_2)
beta_2 = 0
alpha_3 = filter_r*(sum_nr_3/sum_rr_3)
beta_3 = 0

out_file0 = strmid(out_file,0,strlen(out_file)-4)+'_0beta.xdr'
save,file=out_file0,files,f5,filter_r,alpha_0,beta_0,alpha_1,beta_1,alpha_2,beta_2,alpha_3,beta_3

; Now calculate coefficients for reference output only
; No filter is applied here since filter_1 is based on the N/R sampling, which 
; is not a limitation here.
;filter_r[n_elements(filter_r)/4:n_elements(filter_r)/4*3] = 0
alpha_0 = 0
beta_0 = sum_no_0/sum_oo
alpha_1 = 0
beta_1 = sum_no_1/sum_oo
alpha_2 = 0
beta_2 = sum_no_2/sum_oo
alpha_3 = 0
beta_3 = sum_no_3/sum_oo

out_file0 = strmid(out_file,0,strlen(out_file)-4)+'_0alpha.xdr'
save,file=out_file0,files,f5,filter_r,alpha_0,beta_0,alpha_1,beta_1,alpha_2,beta_2,alpha_3,beta_3

end
